const path = require('path')
const { writeFileSync, existsSync, unlinkSync } = require('fs')
const { Logger } = require('./Logger.js')
const { getRepoName } = require('../pre-commit/gitSetup/gitUtils/gitActions.js')
const { getLibraryInstalledLocation } = require('./getLibraryInstalledLocation.js')
const getNodeModulesPath = require('./getNodeModulesPath.js')
const repoName = getRepoName()
const libraryInstalledLocation = getLibraryInstalledLocation()

let eslintConfigurationFileContent = ``
const eslintConfigFilePath = path.join(getNodeModulesPath(),'.eslintrc.js')

function createEslintConfigFile(){
    let pathOfEslintConfigFileInCommonRepo = path.join(libraryInstalledLocation,'configuration','services',repoName,'.eslintrc.js')
    let pathOfServiceSpecificIndexFile = path.join(libraryInstalledLocation,'configuration','services',repoName,'index.js')


    let eslintBasicConfigsRequireStatement = `const { env, parserOptions } = require('${pathOfEslintConfigFileInCommonRepo}')`
    let eslintRulesConfigRequireStatement = `const { rulesConfig, plugins, extendPlugins } = require('${pathOfServiceSpecificIndexFile}')`
    const errorStatus = `const [off,warn,error] = ['off','warn','error']`

    eslintConfigurationFileContent = `
    ${eslintBasicConfigsRequireStatement}
    ${eslintRulesConfigRequireStatement}

    ${errorStatus}

    module.exports = {
    env : env,
    parserOptions : parserOptions,
    extends : extendPlugins,
    plugins : plugins,
    overrides : rulesConfig
    }
    `

    try{
        if(existsSync(eslintConfigFilePath)){
            try{
                unlinkSync(eslintConfigFilePath)
                Logger.log(Logger.SUCCESS_TYPE,'Eslint configuration file removed successfully!')
                return createEslintConfigurationFile()
            }
            catch(err){
                Logger.log(Logger.FAILURE_TYPE,err)
                Logger.log(Logger.FAILURE_TYPE,'Unable to remove the eslint config file.')
                return false
            }
        }
        else{
            return createEslintConfigurationFile()
        }
    }
    catch(error){
        Logger.log(Logger.FAILURE_TYPE,error)
        Logger.log(Logger.FAILURE_TYPE,'Unable to check existense of eslint config file in creatingEslintConfig method')
        return false
    }
}

function createEslintConfigurationFile(){
    try{
        writeFileSync(eslintConfigFilePath,eslintConfigurationFileContent,'utf-8')
        Logger.log(Logger.SUCCESS_TYPE,'Eslint configuration file created successfully!')
        return true
    }
    catch(error){
        Logger.log(Logger.FAILURE_TYPE,error)
        Logger.log(Logger.FAILURE_TYPE,'Unable to create and write an eslint configuration file.')
        return false
    }
}

module.exports = {
    createEslintConfigFile
}






